// $Id: mtrack.cpp,v 1.21 2004/02/21 01:52:31 martin Exp $

// mtrack.cpp, (C) 2004 Martin Henne

#include "mtrack.h"
#include "activity.h"
#include "activitylist.h"
#include "datafile.h"
#include "accountlist.h"
#include "io.h"
#include <qwidget.h>
#include <qtimer.h>
#include <qstring.h>
#include <qsortedlist.h>

QSortedList<Activity> Mtrack::_activityList;
AccountList* Mtrack::_accountList = 0;
const QString Mtrack::_VERSION = "0.1.0pre";
QString Mtrack::_operative = "(unset)";

Mtrack::Mtrack(int argc, char** argv) : _argc(argc), _argv(argv) {
    _saveTimer = new QTimer;
    _timerActive = false;
    connect (_saveTimer, SIGNAL( timeout() ), this, SLOT( saveTimeoutSlot() ) );
}


Mtrack::~Mtrack() {    
    delete _saveTimer;
    _saveTimer = 0;
}


void Mtrack::mtrackStart() {
    const char* envFileName = std::getenv("TEMTOR_DATAFILE");
    if(envFileName) {
	std::fprintf(stderr,"Mtrack::mtrackStart(): std::getenv(\"TEMTOR_DATAFILE\") returned '%s'\n",envFileName);
	_dataFile = new DataFile( QString(envFileName) );
    }
    else
	_dataFile = new DataFile;
    
    _activityList = _dataFile->read();
    _accountList = new AccountList(_activityList);
    _tracking = false;
    if(getCurrentActivity()) {
	//println()("Mtrack::mtrackStart(): found current activity, set tracking TRUE");
	setTracking(true);
    }
    else {
	//println()("Mtrack::mtrackStart(): no current activity, set tracking FALSE");
	setTracking(false);
    }
}


void Mtrack::mtrackEnd() {
    this->save(true);
    
    if(_dataFile) {
	delete _dataFile;
	_dataFile = NULL;
    }
    if(_accountList) {
	delete _accountList;
	_accountList = NULL;
    }
}



void Mtrack::dumpActivityList() {
    Activity* a = (Activity*)( _activityList.first() );
    while (a != NULL) {
	println()(a->toXmlString());
	a = (Activity*)( _activityList.next() );
    }
}


void Mtrack::removeAccount(const QString& name) {
    Activity* a = Mtrack::_activityList.first();
    while(a) {
	QString aName = a->getName();
	if( aName == name ) {
	    std::fprintf(
		stderr,
		"Mtrack::removeAccount(): removing '%s'\n",
		aName.latin1()
		);
	    _activityList.remove();
	    a = _activityList.first();
	}
	else
	    a = _activityList.next();
    }
    Mtrack::_accountList->create(_activityList);
}



void Mtrack::renameProject(const QString& oldName, const QString& newName) {
    Activity* a = Mtrack::_activityList.first();
    while(a) {
	if(a->getName() == oldName) {
	    a->setName(newName);
	}
	a = Mtrack::_activityList.next();
    }
    Mtrack::_accountList->create(_activityList);
}


AccountList* Mtrack::getAccountList() {
    return _accountList;
}


Activity* Mtrack::getCurrentActivity() {
    Activity* a = _activityList.first();
    if(!a) {
	return 0;
    }
    while(a) {
	if(a->isCurrent()) {
	    return a;
	}
	a = _activityList.next();
    }
    return 0;
}


QSortedList<ActivityAccount> Mtrack::getProjectList() {
    QSortedList<QString> names = ActivityList::getProjectNames( Mtrack::_activityList );
    QSortedList<ActivityAccount> list;
    QString* name = names.first();
    while(name) {
	list.append( new ActivityAccount(*name) );
	name = names.next();
    }
    return list;
}


QStringList Mtrack::getProjectNamesList() {
    QSortedList<QString> names = ActivityList::getProjectNames( Mtrack::_activityList );
    QStringList namesList;
    QString* name = names.first();
    while(name) {
	namesList.append( *name );
	name = names.next();
    }
    return namesList;
}


void Mtrack::newCurrentActivity(const QString& name) {
    endCurrentActivity();
    Activity* newCurrent = new Activity(name);
    newCurrent->setCurrent(true);
    _activityList.append(newCurrent);
    setTracking(true);
    _accountList->create(_activityList);
}


void Mtrack::endCurrentActivity() {
    Activity* currentActivity = getCurrentActivity();
    if(!currentActivity) {
	println()("Mtrack::endCurrentActivity(): nothing to end");
	return;
    }
    currentActivity->setEnd( QDateTime::currentDateTime() );
    currentActivity->setCurrent(false);
    setTracking(false);		
    _accountList->create(_activityList);
}


void Mtrack::addActivity(Activity* arg) {
    _activityList.append(arg);
    _accountList->create(_activityList);
}

// end current activity (if any) and create new current activity
void Mtrack::listBoxTapped(int index) {
    std::fprintf(stderr,"Mtrack::listBoxTapped(): called\n");
    QString currentName="";
    QString tappedName = Mtrack::_accountList->getByListBoxIndex(index)->getName();
    Activity* currentActivity = this->getCurrentActivity();
    if( tracking() && currentActivity ) { // we are tracking an activity, end it first
	currentName = currentActivity->getName();
	std::fprintf(stderr,"Mtrack::listBoxTapped(): ending activity '%s'\n",currentName.latin1());
	this->endCurrentActivity();		
	if( currentName == tappedName) {	
	    std::fprintf(stderr,"Mtrack::listBoxTapped(): no new activity to start, returning...\n");
	    return;
	}
    }
    
    // not in tracking mode anymore:
    std::fprintf(stderr,"Mtrack::listBoxTapped(): now starting new activity '%s'\n",tappedName.latin1());
    Activity* newCurrent = new Activity(tappedName);
    newCurrent->setCurrent(true);
    this->setTracking(true);
    Mtrack::_activityList.append(newCurrent);
    Mtrack::_accountList->create(_activityList);
    std::fprintf(stderr,"Mtrack::listBoxTapped(): returning\n");
    return;
}


QString Mtrack::createTimesLine() {
    Activity* currentActivity = getCurrentActivity();
    if(!currentActivity) {
	TimeSpan all = _accountList->getOverallSum();
	return all.toShortString();
    }
    if(!currentActivity->empty()) {
	currentActivity->setEnd(QDateTime::currentDateTime());
    }
    TimeSpan current = currentActivity->getTimeSpan();
    TimeSpan sum = _accountList->getAccountSum( currentActivity->getName() );
    TimeSpan all = _accountList->getOverallSum(*currentActivity);
    QString s;
    s.sprintf(
	"curr(%s)  sum(%s)  all(%s)",
	current.toString().latin1(),
	sum.toShortString().latin1(),
	all.toShortString().latin1()
	);
    return s;
}


void Mtrack::save(bool now) {
    if(now) {
	this->saveTimeoutSlot();
	return;
    }
    if(!_timerActive) {
	_saveTimer->start(500,true);
	_timerActive = true;
    }
}


void Mtrack::exportDataFile(const QString& fileName) {
    if(_dataFile)
	_dataFile->write(_activityList, QString(fileName) );
}


void Mtrack::importDataFile(const QString& fileName, bool) {
    if(_dataFile)
	_activityList = _dataFile->read(fileName);
    update();
}


int Mtrack::getListBoxIndexByName(const QString& name) {
    ActivityAccount* account = _accountList->getByName(name);
    if(account)
	return account->getListBoxIndex();
    
    return -1;
}


QString Mtrack::getNameByListBoxIndex(int index) {
    ActivityAccount* account = Mtrack::_accountList->getByListBoxIndex(index);
    return account->getName();
}


bool Mtrack::tracking() const {
    return _tracking;
}


bool Mtrack::getTracking() const {
    return tracking();
}


void Mtrack::setTracking(bool arg) {
    _tracking = arg;
}


QString Mtrack::getCurrentActivityName() {
    Activity* a = this->getCurrentActivity();
    if(a)
	return a->getName();
    return "";
}


void Mtrack::clearAll() {
    this->endCurrentActivity();
    _accountList->clear();
    _activityList.clear();
}


QString Mtrack::getVersion() {
    return _VERSION;
}


void Mtrack::resetActivity(const QString& name) {
    const QString SUFFIX("--KEEPTHISONE--");
    
    // check if the current activity must end before
    Activity* currentActivity = this->getCurrentActivity();
    if(currentActivity) {
	if(name == currentActivity->getName() ) {
	    this->endCurrentActivity();
	}
    }
    
    // mark first Activity and reset TimeSpan
    Activity* a = Mtrack::_activityList.first();
    while(a) {
	if(a->getName() == name) {
	    a->setName(name+SUFFIX);
	    a->setEnd(a->getStart());
	    break;
	}
	a = _activityList.next();
    }
    
    // remove remaining activities
    a = Mtrack::_activityList.first();
    while(a) {
	if(a->getName() == name) {
	    _activityList.remove();
	    a = _activityList.first();
	}
	a = _activityList.next();
    }
    
    // unmark first Activity
    a = Mtrack::_activityList.first();
    while(a) {
	if(a->getName() == (name+SUFFIX)) {
	    a->setName(name);
	    break;
	}
	a = _activityList.next();
    }
    Mtrack::_accountList->create(_activityList);
}



void Mtrack::resetAll() {
    _accountList->rewind();
    QSortedList<QString> names;
    while( _accountList->hasNext() ) {
	QString* name = new QString(_accountList->getNext()->getName());
	names.append(name);
    }
    QString* nameToReset = names.first();
    while(nameToReset) {
	this->resetActivity(*nameToReset);
	nameToReset = names.next();
    }
    _accountList->create(_activityList);
    std::fprintf(stderr,"Mtrack:resetAll(): reseted '%i' accounts\n",names.count() );
}


QSortedList<Activity>* Mtrack::getActivityList() {
    return &Mtrack::_activityList;
}



Activity* Mtrack::getActivityByIndex(int idx) {
    if(idx < 0) return 0;
    Activity* a = Mtrack::_activityList.first();
    while(a) {
	if( idx == a->getIndex() ) {
	    return a;
	}
	a = Mtrack::_activityList.next();
    }
    return 0;
}



void Mtrack::removeMarked() {
    removeAccount("--REMOVEME--");
}


void Mtrack::update() {
    Mtrack::_accountList->create(_activityList);
}


void Mtrack::setOperative(const QString& arg) {
    _operative = arg;
}


QString Mtrack::getOperative() {
    return _operative;
}


void Mtrack::resetToDate(const QDate& d) {
    Activity* a = _activityList.first();
    while(a) {
	if(a->getStart().date() < d)
	    a->setName("--REMOVEME--");
	a = _activityList.next();
    }
    removeMarked();
}


void Mtrack::resetFromDate(const QDate& d) {
    Activity* a = _activityList.first();
    while(a) {
	if(a->getStart().date() >= d)
	    a->setName("--REMOVEME--");
	a = _activityList.next();
    }
    removeMarked();
}


void Mtrack::resetActivityIndex() {
    Activity* a = _activityList.first();
    while(a) {
	a->setIndex(-1);
	a = _activityList.next();
    }
}


void Mtrack::saveTimeoutSlot() {
    std::fprintf(stderr,"Mtrack::saveTimeoutSlot()\n");
    if(_dataFile)
	_dataFile->write(_activityList);
    _timerActive = false;
}










